/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------


// C Source File
// Created 25/11/02; 11:58:17

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "baseconv.h"

//-----------------------------------------------------------------------------

void InputStr (char *buffer, short maxlen)
{
	SCR_STATE ss;
	short key, i = 0;
	buffer[0] = 0;
	SaveScrState (&ss);
	do
	{
		MoveTo (ss.CurX, ss.CurY);
		printf ("%s_  ", buffer);
		// Note that two spaces are required if F_4x6 font is used
		key = ngetchx ();
		if (key >= ' ' && key <= '~' && i < maxlen) buffer[i++] = key;
		if (key == KEY_BACKSPACE && i) i--;
		buffer[i] = 0;
	} while (key != KEY_ENTER);
}

//-----------------------------------------------------------------------------

int BaseConv(int initial_base, int final_base, char * initial_buffer, char * final_buffer)
{
	int a,b;
	char ch;
	int coef=0;
	
	int correct_digits;
	
	int final_value;
	
	div_t division;
	
	char Upper_Digits[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	char Lower_Digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
	
	int initial_length;
	int final_length;
	
	// ***** incorrect bases *****
	
	if (!(initial_base>=2 && initial_base<=36))
	return 0;
	if (!(final_base>=2 && final_base<=36))
	return 0;
	
	initial_length = strlen(initial_buffer);
	final_value = 0;
	
	// ***** initial_base -> 10 *****
	
	
	for (a=0;a<initial_length;a++)
	{
		ch = initial_buffer[initial_length-a-1];
		
		
		// ***** incorrect Upper_Digits *****
		
		correct_digits = 0;
		
		for (b=0;b<initial_base;b++)
		if (ch==Upper_Digits[b] || ch==Lower_Digits[b])
		{
			coef = b;
			correct_digits = 1;
			break;
		}
		
		if (!correct_digits)
		return 0;
		
		final_value += pow(initial_base,a)*coef;
		
	}
	
	// ***** incorrect Initial Value *****
	
	if (!(final_value>=0 && final_value<32768))
	return 0;
	
	// ***** 10 -> final_base *****
	
	
	final_length = 0;
	
	while (final_value)
	{
		
		division = div(final_value,final_base);
		
		
		memmove(final_buffer+1,final_buffer,final_length);
		final_buffer[0] = Upper_Digits[division.rem];
		
		final_value = division.quot;
		final_length++;
		
	}
	
	final_buffer[final_length] = 0;
	
	
	
	
	return 1;
	
}
//---------------------------------------------------------------------------


// Main Function
void _main(void)
{
	
	char initial_buffer[50];
	char final_buffer[50];
	
	char buffer[50];
	
	int initial_base;
	int final_base;
	
	
	FontSetSys(0);
	clrscr();
	printf("Base Converter 1.2 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	FontSetSys(0);
	
	for (;;)
	{
		
		clrscr();
		
		printf ("Initial Value : \n");
		InputStr(initial_buffer,32);
		printf("\n\n");
		
		printf ("Initial Base : \n");
		InputStr(buffer,32);
		printf("\n\n");
		
		initial_base = atoi(buffer);
		
		printf ("Final Base : \n");
		InputStr(buffer,32);
		printf("\n\n");
		
		final_base = atoi(buffer);
		
		printf ("Final Value : \n");
		
		if (BaseConv(initial_base,final_base,initial_buffer,final_buffer))
		printf ("%s",final_buffer);
		else
		printf ("Error!");
		
		printf("\n\n");
		printf("[ESC] : Exit\n");
		
		if (ngetchx()==KEY_ESC)
		break;
		
	}
	
}
